"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const api_service_core_1 = require("api-service-core");
// import { Request, Response, NextFunction } from "express";
const _ = require("lodash");
const sanitizeHtml = require("sanitize-html");
const dexss_rules_1 = require("./dexss.rules");
/**
 * TODO: replace with https://www.npmjs.com/package/sanitize-html
 */
class DeXSS {
    constructor() {
        this.name = "api.dexss";
        this.title = "Sanitize incoming data - removing HTML/Scripts";
    }
    fn(_op, _options) {
        this.options = _.extend({ rules: dexss_rules_1.default, deep: true, fields: "*" }, _options);
        return (req, _res, next) => {
            _op.context.log({
                code: "api:dexss:sanitize",
                type: api_service_core_1.Vars.typeof(req.body),
                model: _.keys(req.body),
            });
            if (!this.options.fields || this.options.fields == "*") {
                this.sanitizeAllFields(req.body);
            }
            else {
                this.sanitizeFields(this.options.fields, req.body);
            }
            _op.context.log({
                code: "api:dexss:sanitized",
                type: api_service_core_1.Vars.typeof(req.body),
            });
            next();
        };
    }
    sanitizeFields(fields, _model) {
        let models = _.isArray(_model) ? _model : [_model];
        for (let m in models) {
            let model = models[m];
            for (let f in fields) {
                model[f] = sanitizeHtml(model[f], this.options.rules);
            }
        }
        return _model;
    }
    sanitizeAllFields(_model) {
        let models = _.isArray(_model) ? _model : [_model];
        for (let m in models) {
            let model = models[m];
            for (let f in model) {
                let field = model[f];
                if (_.isString(field)) {
                    // console.log("field.string: %j -> %j", f, field);
                    model[f] = sanitizeHtml(field, this.options.rules);
                }
                else if (this.options.deep && _.isArray(field)) {
                    let items = [];
                    for (let i in field) {
                        let item = field[i];
                        // console.log("field.array: %j -> %j -> %j", f, i, item);
                        if (typeof item === "string")
                            items.push(sanitizeHtml(item, this.options.rules));
                        else
                            items.push(this.sanitizeAllFields(item));
                    }
                }
                else if (this.options.deep && _.isObject(field)) {
                    // console.log("field.array: %j -> %j", f, field);
                    model[f] = this.sanitizeAllFields(model[f]);
                }
            }
        }
        return _model;
    }
}
exports.default = DeXSS;
//# sourceMappingURL=dexss.js.map