"use strict";
/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const api_service_core_1 = require("api-service-core");
// const request = require("request");
const exphbs = require("express-handlebars");
const Handlebars = require("handlebars");
/**
 * View CMS Page
 * -------------
 * A simple "micro-CMS" that renders a Handlebars page based on static meta-data and optional dynamic API call.
 *
 * The same JWT used for this UI layer will be forwarded to the API endpoint (unless reuseJWT = true).
 *
 * Bindings can be injected from config or options -
 *      static:
 *      brand:
 *          title: Site / App name
 *          description: Generic site description
 *      apis:
 *          my_app:
 *              url: request-url
 *              headers: request-headers
 *              etc
 *      feature:
 *          a: feature toggle A
 *          b: toggle B
 *
 * @type {{name: string, title: string, description: string, _render: module.exports._render, fn: module.exports.fn}}
 */
class render {
    constructor() {
        this.name = "render";
        this.title = "Convenient server-side rendering";
    }
    install(context, _options) {
        let options = _options;
        // template rendering options
        this.render_options = _.extend({
            defaultLayout: "default",
            layoutsDir: "views/layouts",
            partialsDir: "views/partials",
            extname: ".hbs",
        }, options);
        // create the Handlebars renderer
        this.hbs = exphbs.create(this.render_options);
        // attach render to engine
        context.app.engine(".hbs", this.hbs.engine);
        context.app.set("view engine", ".hbs");
        // site-wide properties
        Handlebars.registerHelper("json", function (data) {
            return JSON.stringify(data);
        });
        Handlebars.registerHelper("html", function (text) {
            let replaced = text
                ? text.replace(new RegExp("\r?\n", "g"), "<br/>")
                : "";
            return replaced;
        });
        context.middleware.set(new RenderView(context, _options));
    }
}
exports.default = render;
// interface IViewRequest {
// }
class RenderView {
    constructor(context, options) {
        this.name = "api.render";
        this.context = context;
        this.options = options;
    }
    /**
     * Collect a set of API responses as promise
     *
     * @param apis
     * @param options
     * @param token
     */
    // _requests(apis: any, options: any, token: string): Promise<any> {
    //     let _requests = [];
    //     // iterate over set of APIs
    //     for (let api_name in apis) {
    //         let api = apis[api_name];
    //         let _headers: any = {};
    //         if (token) {
    //             // inject Bearer Token
    //             _headers.authorization = token;
    //         }
    //         let api_url = Vars.$(api.url, options);
    //         let _request = this.backend_request( api_name, api_url, _headers);
    //         _requests.push( _request );
    //     }
    //     return Promise.all(_requests);
    // }
    // backend_request( api_name: string, url: string, _headers: any ): Promise<any> {
    //     let context = this.context;
    //     return new Promise(( resolve, reject ) => {
    //         // call backend API
    //         context.log({ code: "plugin:render:api", api: api_name, url: url, headers: _headers });
    //         request({ uri: url, headers: _headers }, function (err: any, httpResponse: any, body: any) {
    //             if (err || !httpResponse) {
    //                 context.error({ code: "plugin:render:api:failed", api: api_name, url: url, httpResponse: httpResponse?true:false, error: err?err.toString():"no error" });
    //                 reject(err);
    //             } else {
    //                 try {
    //                     let result = {};
    //                     result[api_name] = JSON.parse(body);
    //                     resolve( result );
    //                 } catch (e) {
    //                     reject(e);
    //                 }
    //             }
    //         });
    //     });
    // }
    fn(operation, _options) {
        let options = _.extend({}, this.options, _options);
        let context = operation.context;
        let feature_apis = _.extend({}, this.options.apis, options.apis);
        // let is_static = (_.isEmpty(feature_apis));
        // console.log("OPTIONS: %j", _options);
        let page_defaults = {
            brand: _.extend({}, this.options.brand, options.brand),
            i18n: _.extend({}, this.options.i18n, options.i18n),
            pkg: { name: context.pkg.name, version: context.pkg.version },
        };
        context.log({
            code: "plugin:render",
            resource: operation.resource,
            reuseJWT: options.reuseJWT,
            page: page_defaults,
            apis: _.keys(feature_apis),
            pkg: page_defaults.pkg,
        });
        // renderer middleware
        return (req, res) => {
            let jwt_token = options.reuseJWT ? req.header("authorization") : "";
            let req_params = _.extend({}, {}, options.params, req.params);
            // parameters for .hbs render
            let page_context = _.extend({}, options.render, { params: req_params, query: req.query, url: req.url }, page_defaults);
            // prefer 'page' from request, then options, otherwise 'index'
            let page = req.params.page || req.params[0] || options.page || "index";
            let ix = page.lastIndexOf(".");
            if (ix > 0) {
                page = page.substring(0, ix);
            }
            page = page + ".hbs";
            page_context.params.page = page;
            let page_path = (page_context.params.path
                ? page_context.params.path + "/"
                : "") + page;
            page_context.layout =
                options.layout || page_context.params.layout || "default";
            // console.log("RENDER: (%s) %s -> %j -> %j", page_context.layout, page_path, req.params, page_context);
            // parameters for API requests
            let api_params = _.extend({}, options.params, req_params, page_context);
            // get the API responses ...
            context.log({
                code: "plugin:render:apis",
                apis: _.keys(feature_apis),
                params: api_params,
            });
            api_service_core_1.APIs.requests(this.context, feature_apis, api_params, jwt_token)
                .then((results) => {
                // merge API responses
                page_context = _.extend(page_context, results);
                // render page response
                try {
                    res.render(page_path, page_context);
                    context.log({
                        code: "plugin:render:apis:page",
                        page: page_path,
                        layout: page_context.layout,
                        context: _.keys(page_context),
                    });
                }
                catch (e) {
                    console.error("plugin:render:apis:page:failed: %j", e);
                    res.status(404);
                    res.send({
                        code: "plugin:render:apis:page:failed",
                        error: e.message,
                    });
                }
            })
                .catch((err) => {
                console.error("plugin:render:apis:failed: %j", err);
                // render error response
                let _error = {
                    code: "plugin:render:apis:failed",
                    page: page_path,
                    layout: page_context.layout,
                    message: err.message,
                    error: err.message,
                };
                context.error(_error);
                res.status(500);
                res.send({
                    code: "plugin:render:apis:failed",
                    request: req.a6s_uuid,
                });
            });
        };
    }
}
//# sourceMappingURL=render.js.map