/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */


import * as _ from "lodash";

import {IChassisMiddleware, IOperation, APIs, Vars }  from "api-service-core";
import Axios, { AxiosRequestConfig, AxiosResponse } from "axios";

export default class APIMiddleware implements IChassisMiddleware {
    name = "api";

    fn(operation: IOperation, _options: any): Function {

        let options = _.extend({ params: {}, reuseJWT: false, headers: {} }, _options);
        let context = operation.context;

        // forward request to remote API ...
        return (req, res) => {

            let ctx = APIs.getOperationContext(operation, req);
            let jwt_token = options.reuseJWT?req.header("authorization"):'';
            let headers = APIs.get_headers_with_token( _.extend({}, req.headers, options.headers), jwt_token);
            let api_url = Vars.$(options.url, ctx);
            delete headers.host;
            // parameters for API request
            let api_req = { url: api_url, method: req.method, params: req.params, headers: headers, data: req.body } as AxiosRequestConfig;
            context.log({ code: "api:request", api_req: api_req });

            Axios.request( api_req )
                .then((rep: AxiosResponse) => {
                    res.status(rep.status);
                    res.send(rep.data);
                })
                .catch((err) => {
                    return APIs.oops(context, res, {
                        code: "api:request:failed",
                        statusCode: err.response?err.response.status: 500,
                        api_req: api_req,
                        error: err ? err.message : "no error",
                    });
                });
        }
    }
}
