/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */


import * as _ from "lodash";

import {IChassisMiddleware, IOperation, APIs, APIRequests }  from "api-service-core";

export default class APIsMiddleware implements IChassisMiddleware {
    name = "apis";

    fn(operation: IOperation, _options: any): Function {

        let context = operation.context;
        let feature_apis: APIRequests = _options;

        // APIs middleware
        return (req, res) => {

            // parameters for API requests
            let ctx = APIs.getOperationContext(operation,req,{});
            let jwt_token = req.header("authorization") || "";

            context.log({ code: "plugin:apis:requests", apis: _.keys(feature_apis), ctx: ctx });

            // get the API responses ...
            APIs.requests(context, feature_apis, ctx, jwt_token).then(results=> {

                // APIs response
                res.status(200).send(results);
                context.log({ code: "plugin:apis:results", results: _.keys(results), context: _.keys(ctx) });

            }).catch(err => {
                // APIs error response
                let _error = { code: "plugin:apis:failed", message: err.message, error: err.toString() };
                context.error(_error);
                res.status(500).send(_error);
            });

        }
    }
}
