/*************************************************************************
 *
 * Troven CONFIDENTIAL
 * __________________
 *
 *  (c) 2017-2020 Troven Ventures Pty Ltd
 *  All Rights Reserved.
 *
 * NOTICE:  All information contained herein is, and remains
 * the property of Troven Pty Ltd and its licensors,
 * if any.  The intellectual and technical concepts contained
 * herein are proprietary to Troven Pty Ltd
 * and its suppliers and may be covered by International and Regional Patents,
 * patents in process, and are protected by trade secret or copyright law.
 * Dissemination of this information or reproduction of this material
 * is strictly forbidden unless prior written permission is obtained
 * from Troven Pty Ltd.
 */

import { IOperation, IChassisMiddleware, Vars } from "api-service-core";
// import { Request, Response, NextFunction } from "express";
import * as _ from "lodash";
import * as sanitizeHtml from "sanitize-html";
import dexss_rules from "./dexss.rules";

/**
 * TODO: replace with https://www.npmjs.com/package/sanitize-html
 */
export default class DeXSS implements IChassisMiddleware {
    name = "api.dexss";
    title = "Sanitize incoming data - removing HTML/Scripts";
    options: any;

    fn(_op: IOperation, _options: any): Function {
        this.options = _.extend(
            { rules: dexss_rules, deep: true, fields: "*" },
            _options
        );

        return (req, _res, next) => {
            _op.context.log({
                code: "api:dexss:sanitize",
                type: Vars.typeof(req.body),
                model: _.keys(req.body),
            });

            if (!this.options.fields || this.options.fields == "*") {
                this.sanitizeAllFields(req.body);
            } else {
                this.sanitizeFields(this.options.fields, req.body);
            }
            _op.context.log({
                code: "api:dexss:sanitized",
                type: Vars.typeof(req.body),
            });
            next();
        };
    }

    sanitizeFields(fields: [], _model: any): any {
        let models = _.isArray(_model)?_model:[_model];
        for(let m in models) {
            let model = models[m]
            for (let f in fields) {
                model[f] = sanitizeHtml(model[f], this.options.rules);
            }
        }
        return _model;
    }

    sanitizeAllFields(_model: any): any {
        let models = _.isArray(_model)?_model:[_model];
        for(let m in models) {
            let model = models[m]
            for (let f in model) {
                let field = model[f];
                if (_.isString(field)) {
// console.log("field.string: %j -> %j", f, field);
                    model[f] = sanitizeHtml(field, this.options.rules);
                } else if ( this.options.deep && _.isArray(field) ) {
                    let items = [];
                    for(let i in field) {
                        let item = field[i];
// console.log("field.array: %j -> %j -> %j", f, i, item);
                        if (typeof item === "string")
                            items.push( sanitizeHtml(item, this.options.rules) );
                        else
                            items.push( this.sanitizeAllFields(item) );
                    }
                } else if ( this.options.deep && _.isObject(field) ) {
// console.log("field.array: %j -> %j", f, field);
                    model[f] = this.sanitizeAllFields(model[f]);
                }
            }
        }
        return _model;
    }
}
