import {IChassisContext, IChassisPlugin } from "api-service-core";

import * as helmet from 'helmet';
import * as _ from "lodash"

/**
 * Harden
 * ------
 * Feature to protect micro-service from request-side abuse
 * Essentially configures Helmet:
 * 
 * For options, see: https://helmetjs.github.io/docs/
 *
 *  {
     *      harden:
     *          helmet-options
     *  }
 * @param context
 * @param options
 * @returns {exports}
 */

export default class harden implements IChassisPlugin {

    name = "harden";
    title = "harden requests with Helmet";

    install(context: IChassisContext, options: any) {

        context.app.disable('x-powered-by');
        context.app.use( helmet( options ) );

        context.log({ code: "api:plugin:harden", message: "chassis hardened", options: _.keys(options) });
    }

}